unit Qcopy;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Consts,DsgnIntf;

type
  TCommand = (cmYes, cmNo);
  TOverwrite = (owAlways, owAsk, owNever);
  TOverwriteEvent = procedure(const FileName: String; var Command: TCommand) of object;
  TFileCopyEvent = procedure(FileSize, BytesCopied: Longint) of object;
  TQuickCopy = class(TComponent)
  private
    { Private declarations }
  fOnFileCopy: TFileCopyEvent;
  fOnCopyDone: TNotifyEvent;
  FOverwrite: TOverwrite;
  FOnOverwrite: TOverwriteEvent;
  FAbout:	string;
  procedure SetOverwrite(Value: TOverwrite);
  protected
    { Protected declarations }

  public
    { Public declarations }
  constructor Create(AOwner: TComponent);
  procedure   Execute(const FileName, DestName: TFileName);
  procedure ShowAbout;
  published
    { Published declarations }
  property OnFileCopy: TFileCopyEvent read fOnFileCopy write fOnFileCopy;
  property OnCopyDone: TNotifyEvent read fOnCopyDone write fOnCopyDone;
  property Overwrite: TOverwrite read FOverwrite write SetOverwrite stored True;
  property OnOverwrite: TOverwriteEvent read FOnOverwrite write FOnOverwrite;
  property About: string read FAbout write FAbout stored False;
  end;

procedure Register;

implementation
type
  TAboutProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue:string; override;
  end;

procedure TAboutProperty.Edit;
{Invoke the about dialog when clicking on ... in the Object Inspector}
begin
  TQuickCopy(GetComponent(0)).ShowAbout;
end;

function TAboutProperty.GetAttributes: TPropertyAttributes;
{Make settings for just displaying a string in the ABOUT property in the
Object Inspector}
begin
  GetAttributes := [paDialog, paReadOnly];
end;


function TAboutProperty.GetValue: String;
{Text in the Object Inspector for the ABOUT property}
begin
  GetValue := '(About)';
end;


procedure TQuickCopy.ShowAbout;
var
	msg: string;
const
	carriage_return = chr(13);
  copyright_symbol = chr(169);
begin
  Msg := 'QuickCopy 1.01';
  AppendStr(msg, carriage_return);
  AppendStr(msg, carriage_return);
  AppendStr(msg, carriage_return);
  AppendStr(msg, 'Copyright ');
  AppendStr(msg, copyright_symbol);
  AppendStr(msg, ' 1994 - 1997 by QuickSoft');
  AppendStr(msg, carriage_return);
  ShowMessage(msg);
end;



const
  SInvalidDest = 'Destination %s does not exist';
  SFCantMove = 'Cannot move file %s';

function HasAttr(const FileName: string; Attr: Word): Boolean;
begin
  Result := (FileGetAttr(FileName) and Attr) = Attr;
end;

function GetFileSize(const FileName: string): LongInt;
var
  SearchRec: TSearchRec;
begin
  if FindFirst(ExpandFileName(FileName), faAnyFile, SearchRec) = 0 then
    Result := SearchRec.Size
  else Result := -1;
end;

constructor TQuickCopy.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FOverwrite:=owAsk;                    { Default 'owAsk' }
end;

procedure TQuickCopy.SetOverwrite(Value: TOverwrite);
begin
  if Value<>FOverwrite then FOverwrite:=Value;
end;

procedure TQuickCopy.Execute(const FileName, DestName: TFileName);
var
  CopyBuffer: Pointer; { buffer for copying }
  TimeStamp,
  FileSize,
  BytesCopied,
  Copied: Longint;
  Source, Dest: Integer; { handles }
  Destination: TFileName; { holder for expanded destination name }
  Cmd:TCommand;
const
  ChunkSize: Longint = 8192; { copy in 8K chunks }
begin
  Destination := ExpandFileName(DestName); { expand the destination path }
  if HasAttr(Destination, faDirectory) then { if destination is a directory... }
    Destination := Destination + '\' + ExtractFileName(FileName); { ...clone file name }
  TimeStamp := FileAge(FileName); { get source's time stamp }
  FileSize :=GetFileSize(FileName);
  GetMem(CopyBuffer, ChunkSize); { allocate the buffer }
  Copied:=0;
  Cmd:=cmYes;  { Default return = Overwrite }
  if (FOverwrite=owAsk)and (FileExists(Destination))and
     (Assigned(FOnOverwrite)) then FOnOverwrite(Destination,Cmd);
  if (not FileExists(Destination))or(FOverwrite=owAlways)or
   ((FOverwrite=owAsk)and(Cmd=cmYes)) then
   begin

  try
    Source := FileOpen(FileName, fmShareDenyWrite); { open source file }
    if Source < 0 then raise EFOpenError.Create(FmtLoadStr(SFOpenError, [FileName]));
    try
      Dest := FileCreate(Destination); { create output file; overwrite existing }
      if Dest < 0 then raise EFCreateError.Create(FmtLoadStr(SFCreateError, [Destination]));
      try
        repeat
          BytesCopied := FileRead(Source, CopyBuffer^, ChunkSize); { read chunk }
          if BytesCopied > 0 then { if we read anything... }
           begin
            FileWrite(Dest, CopyBuffer^, BytesCopied); { ...write chunk }
            Inc(Copied,BytesCopied);
         if Assigned(fOnFileCopy) then FOnFileCopy(FileSize,Copied);
           end;
        until BytesCopied < ChunkSize; { until we run out of chunks }
      finally
        FileClose(Dest); { close the destination file }
{        SetFileTimeStamp(Destination, TimeStamp);} { clone source's time stamp }{!!!}
      end;
    finally
      FileClose(Source); { close the source file }
    end;
  finally
    FreeMem(CopyBuffer, ChunkSize); { free the buffer }
  if Assigned(fOnCopyDone) then fOnCopyDone(Self);
  end;
 end;
end;


procedure Register;
begin
  RegisterComponents('Samples', [TQuickCopy]);
  RegisterPropertyEditor(TypeInfo(String), TQuickCopy, 'About',
  TAboutProperty);
end;

end.
